@@ -38,6 +38,7 @@
  * </example>
  */
 
+#include <arvconfig.h>
 #include <arvcamera.h>
 #include <arvsystem.h>
 #include <arvgvinterface.h>
@@ -62,8 +63,9 @@
  * @ARV_CAMERA_VENDOR_BASLER: Basler
  * @ARV_CAMERA_VENDOR_PROSILICA: Prosilica
  * @ARV_CAMERA_VENDOR_TIS: The Imaging Source
- * @ARV_CAMERA_VENDOR_POINT_GREY: PointGrey
- * @ARV_CAMERA_VENDOR_XIMEA: XIMEA Gmbh
+ * @ARV_CAMERA_VENDOR_POINT_GREY_FLIR: PointGrey / FLIR
+ * @ARV_CAMERA_VENDOR_XIMEA: XIMEA GmbH
+ * @ARV_CAMERA_VENDOR_MATRIX_VISION: Matrix Vision GmbH
  */
 
 typedef enum {
@@ -72,9 +74,10 @@ typedef enum {
 	ARV_CAMERA_VENDOR_DALSA,
 	ARV_CAMERA_VENDOR_PROSILICA,
 	ARV_CAMERA_VENDOR_TIS,
-	ARV_CAMERA_VENDOR_POINT_GREY,
+	ARV_CAMERA_VENDOR_POINT_GREY_FLIR,
 	ARV_CAMERA_VENDOR_RICOH,
-	ARV_CAMERA_VENDOR_XIMEA
+	ARV_CAMERA_VENDOR_XIMEA,
+	ARV_CAMERA_VENDOR_MATRIX_VISION
 } ArvCameraVendor;
 
 typedef enum {
@@ -85,9 +88,10 @@ typedef enum {
 	ARV_CAMERA_SERIES_DALSA,
 	ARV_CAMERA_SERIES_PROSILICA,
 	ARV_CAMERA_SERIES_TIS,
-	ARV_CAMERA_SERIES_POINT_GREY,
+	ARV_CAMERA_SERIES_POINT_GREY_FLIR,
 	ARV_CAMERA_SERIES_RICOH,
-	ARV_CAMERA_SERIES_XIMEA
+	ARV_CAMERA_SERIES_XIMEA,
+	ARV_CAMERA_SERIES_MATRIX_VISION
 } ArvCameraSeries;
 
 static GObjectClass *parent_class = NULL;
@@ -910,19 +914,20 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate)
 			} else
 				arv_device_set_float_feature_value (camera->priv->device, "FPS", frame_rate);
 			break;
-		case ARV_CAMERA_VENDOR_POINT_GREY:
+		case ARV_CAMERA_VENDOR_POINT_GREY_FLIR:
 			arv_device_set_string_feature_value (camera->priv->device, "TriggerSelector", "FrameStart");
 			arv_device_set_string_feature_value (camera->priv->device, "TriggerMode", "Off");
 			if (camera->priv->has_acquisition_frame_rate_enabled)
 				arv_device_set_integer_feature_value (camera->priv->device, "AcquisitionFrameRateEnabled", 1);
 			else
-				arv_device_set_integer_feature_value (camera->priv->device, "AcquisitionFrameRateEnable", 1);
+				arv_device_set_boolean_feature_value (camera->priv->device, "AcquisitionFrameRateEnable", 1);
 			arv_device_set_string_feature_value (camera->priv->device, "AcquisitionFrameRateAuto", "Off");
 			arv_device_set_float_feature_value (camera->priv->device, "AcquisitionFrameRate", frame_rate);
 			break;
 		case ARV_CAMERA_VENDOR_DALSA:
 		case ARV_CAMERA_VENDOR_RICOH:
 		case ARV_CAMERA_VENDOR_XIMEA:
+		case ARV_CAMERA_VENDOR_MATRIX_VISION:
 		case ARV_CAMERA_VENDOR_UNKNOWN:
 			arv_device_set_string_feature_value (camera->priv->device, "TriggerSelector", "FrameStart");
 			arv_device_set_string_feature_value (camera->priv->device, "TriggerMode", "Off");
@@ -966,11 +971,12 @@ arv_camera_get_frame_rate (ArvCamera *camera)
 					return 0;
 			} else
 				return arv_device_get_float_feature_value (camera->priv->device, "FPS");
-		case ARV_CAMERA_VENDOR_POINT_GREY:
+		case ARV_CAMERA_VENDOR_POINT_GREY_FLIR:
 		case ARV_CAMERA_VENDOR_DALSA:
 		case ARV_CAMERA_VENDOR_RICOH:
 		case ARV_CAMERA_VENDOR_BASLER:
 	        case ARV_CAMERA_VENDOR_XIMEA:
+		case ARV_CAMERA_VENDOR_MATRIX_VISION:
 	        case ARV_CAMERA_VENDOR_UNKNOWN:
 			return arv_device_get_float_feature_value (camera->priv->device,
 								   camera->priv->has_acquisition_frame_rate ?
@@ -1033,11 +1039,12 @@ arv_camera_get_frame_rate_bounds (ArvCamera *camera, double *min, double *max)
 		case ARV_CAMERA_VENDOR_PROSILICA:
 			arv_device_get_float_feature_bounds (camera->priv->device, "AcquisitionFrameRateAbs", min, max);
 			break;
-		case ARV_CAMERA_VENDOR_POINT_GREY:
+		case ARV_CAMERA_VENDOR_POINT_GREY_FLIR:
 		case ARV_CAMERA_VENDOR_DALSA:
 		case ARV_CAMERA_VENDOR_RICOH:
 		case ARV_CAMERA_VENDOR_BASLER:
 	        case ARV_CAMERA_VENDOR_XIMEA:
+		case ARV_CAMERA_VENDOR_MATRIX_VISION:
 		case ARV_CAMERA_VENDOR_UNKNOWN:
 			arv_device_get_float_feature_bounds (camera->priv->device,
 							     camera->priv->has_acquisition_frame_rate ?
@@ -1070,7 +1077,7 @@ arv_camera_set_trigger (ArvCamera *camera, const char *source)
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 	g_return_if_fail (source != NULL);
 
-	if (camera->priv->vendor ==  ARV_CAMERA_VENDOR_BASLER)
+	if (camera->priv->vendor == ARV_CAMERA_VENDOR_BASLER)
 		arv_device_set_integer_feature_value (camera->priv->device, "AcquisitionFrameRateEnable", 0);
 
 	arv_device_set_string_feature_value (camera->priv->device, "TriggerSelector",
@@ -1241,6 +1248,11 @@ arv_camera_set_exposure_time (ArvCamera *camera, double exposure_time_us)
 			arv_device_set_integer_feature_value (camera->priv->device, "ExposureTime",
 							      exposure_time_us);
 			break;
+		case ARV_CAMERA_SERIES_MATRIX_VISION:
+			arv_device_set_string_feature_value (camera->priv->device, "ExposureMode", "Timed");
+			arv_device_set_float_feature_value (camera->priv->device, "ExposureTime",
+							      exposure_time_us);
+			break;
 		case ARV_CAMERA_SERIES_BASLER_ACE:
 		default:
 			arv_device_set_float_feature_value (camera->priv->device,
@@ -1554,11 +1566,12 @@ arv_camera_is_frame_rate_available (ArvCamera *camera)
 			return arv_device_get_feature (camera->priv->device, "AcquisitionFrameRateAbs") != NULL;
 		case ARV_CAMERA_VENDOR_TIS:
 			return arv_device_get_feature (camera->priv->device, "FPS") != NULL;
-		case ARV_CAMERA_VENDOR_POINT_GREY:
+		case ARV_CAMERA_VENDOR_POINT_GREY_FLIR:
 		case ARV_CAMERA_VENDOR_DALSA:
 		case ARV_CAMERA_VENDOR_RICOH:
 		case ARV_CAMERA_VENDOR_BASLER:
 	        case ARV_CAMERA_VENDOR_XIMEA:
+		case ARV_CAMERA_VENDOR_MATRIX_VISION:
 		case ARV_CAMERA_VENDOR_UNKNOWN:
 			return arv_device_get_feature (camera->priv->device,
 						       camera->priv->has_acquisition_frame_rate ?
@@ -1661,10 +1674,23 @@ arv_camera_is_gain_auto_available (ArvCamera *camera)
 gboolean
 arv_camera_is_binning_available (ArvCamera *camera)
 {
+	ArvGcNode* node;
+
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), FALSE);
 
-	return arv_device_get_feature (camera->priv->device, "BinningHorizontal") != NULL &&
-	       arv_device_get_feature (camera->priv->device, "BinningVertical") != NULL;
+	node = arv_device_get_feature (camera->priv->device, "BinningHorizontal");
+	if (!ARV_IS_GC_FEATURE_NODE (node))
+		return FALSE;
+	if (!arv_gc_feature_node_is_available (ARV_GC_FEATURE_NODE (node), NULL))
+		return FALSE;
+
+	node = arv_device_get_feature (camera->priv->device, "BinningVertical");
+	if (!ARV_IS_GC_FEATURE_NODE (node))
+		return FALSE;
+	if (!arv_gc_feature_node_is_available (ARV_GC_FEATURE_NODE (node), NULL))
+		return FALSE;
+
+	return TRUE;
 }
 
 /**
@@ -2230,15 +2256,18 @@ arv_camera_constructor (GType gtype, guint n_properties, GObjectConstructParam *
 	} else if (g_strcmp0 (vendor_name, "DALSA") == 0) {
 		vendor = ARV_CAMERA_VENDOR_DALSA;
 		series = ARV_CAMERA_SERIES_DALSA;
-	} else if (g_strcmp0 (vendor_name, "Point Grey Research") == 0) {
-		vendor = ARV_CAMERA_VENDOR_POINT_GREY;
-		series = ARV_CAMERA_SERIES_POINT_GREY;
+	} else if (g_strcmp0 (vendor_name, "Point Grey Research") == 0 || g_strcmp0 (vendor_name, "FLIR") == 0) {
+		vendor = ARV_CAMERA_VENDOR_POINT_GREY_FLIR;
+		series = ARV_CAMERA_SERIES_POINT_GREY_FLIR;
 	} else if (g_strcmp0 (vendor_name, "Ricoh Company, Ltd.") == 0) {
 		vendor = ARV_CAMERA_VENDOR_RICOH;
 		series = ARV_CAMERA_SERIES_RICOH;
 	} else if (g_strcmp0 (vendor_name, "XIMEA GmbH") == 0) {
 		vendor = ARV_CAMERA_VENDOR_XIMEA;
 		series = ARV_CAMERA_SERIES_XIMEA;
+	} else if (g_strcmp0 (vendor_name, "MATRIX VISION GmbH") == 0) {
+		vendor = ARV_CAMERA_VENDOR_MATRIX_VISION;
+		series = ARV_CAMERA_SERIES_MATRIX_VISION;
 	} else {
 		vendor = ARV_CAMERA_VENDOR_UNKNOWN;
 		series = ARV_CAMERA_SERIES_UNKNOWN;
@@ -2294,7 +2323,9 @@ arv_camera_class_init (ArvCameraClass *camera_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (camera_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (camera_class, sizeof (ArvCameraPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (camera_class);
 
@@ -2312,4 +2343,8 @@ arv_camera_class_init (ArvCameraClass *camera_class)
 							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvCamera, arv_camera, G_TYPE_OBJECT)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvCamera, arv_camera, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvCamera))
+#endif
