@@ -48,6 +48,11 @@ typedef enum {
 	ARV_GC_CACHABLE_WRITE_AROUND
 } ArvGcCachable;
 
+typedef enum {
+	ARV_GC_SIGNEDNESS_SIGNED,
+	ARV_GC_SIGNEDNESS_UNSIGNED
+} ArvGcSignedness;
+
 /**
  * ArvAuto:
  * @ARV_AUTO_OFF: manual setting
@@ -114,15 +119,30 @@ typedef guint32 ArvPixelFormat;
 #define ARV_PIXEL_FORMAT_BAYER_GB_12		0x01100012
 #define ARV_PIXEL_FORMAT_BAYER_BG_12		0x01100013
 
+#define ARV_PIXEL_FORMAT_BAYER_GR_16		0x0110002e
+#define ARV_PIXEL_FORMAT_BAYER_RG_16		0x0110002f
+#define ARV_PIXEL_FORMAT_BAYER_GB_16		0x01100030
+#define ARV_PIXEL_FORMAT_BAYER_BG_16		0x01100031
+
+#define ARV_PIXEL_FORMAT_BAYER_BG_10P		0x010a0052
+#define ARV_PIXEL_FORMAT_BAYER_GB_10P		0x010a0054
+#define ARV_PIXEL_FORMAT_BAYER_GR_10P		0x010a0056
+#define ARV_PIXEL_FORMAT_BAYER_RG_10P		0x010a0058
+
+#define ARV_PIXEL_FORMAT_BAYER_BG_12P		0x010c0053
+#define ARV_PIXEL_FORMAT_BAYER_GB_12P		0x010c0055
+#define ARV_PIXEL_FORMAT_BAYER_GR_12P		0x010c0057
+#define ARV_PIXEL_FORMAT_BAYER_RG_12P		0x010c0059
+
 #define ARV_PIXEL_FORMAT_BAYER_GR_12_PACKED	0x010c002a
 #define ARV_PIXEL_FORMAT_BAYER_RG_12_PACKED	0x010c002b
 #define ARV_PIXEL_FORMAT_BAYER_GB_12_PACKED	0x010c002c
 #define ARV_PIXEL_FORMAT_BAYER_BG_12_PACKED	0x010c002d
 
-#define ARV_PIXEL_FORMAT_BAYER_GR_16		0x0110002e
-#define ARV_PIXEL_FORMAT_BAYER_RG_16		0x0110002f
-#define ARV_PIXEL_FORMAT_BAYER_GB_16		0x01100030
-#define ARV_PIXEL_FORMAT_BAYER_BG_16		0x01100031
+#define ARV_PIXEL_FORMAT_BAYER_GR_10_PACKED	0x010c0026
+#define ARV_PIXEL_FORMAT_BAYER_RG_10_PACKED	0x010c0027
+#define ARV_PIXEL_FORMAT_BAYER_GB_10_PACKED	0x010c0028
+#define ARV_PIXEL_FORMAT_BAYER_BG_10_PACKED	0x010c0029
 
 /* Color pixel formats */
 
