@@ -51,11 +51,8 @@ struct _ArvFakeCameraPrivate {
 	guint32 frame_id;
 	double trigger_frequency;
 
-#if GLIB_CHECK_VERSION(2,32,0)
 	GMutex fill_pattern_mutex;
-#else
-	GMutex *fill_pattern_mutex;
-#endif
+
 	ArvFakeCameraFillPattern fill_pattern_callback;
 	void *fill_pattern_data;
 };
@@ -251,11 +248,8 @@ arv_fake_camera_set_fill_pattern (ArvFakeCamera *camera,
 {
 	g_return_if_fail (ARV_IS_FAKE_CAMERA (camera));
 
-#if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_lock (&camera->priv->fill_pattern_mutex);
-#else
-	g_mutex_lock (camera->priv->fill_pattern_mutex);
-#endif
+
 	if (fill_pattern_callback != NULL) {
 		camera->priv->fill_pattern_callback = fill_pattern_callback;
 		camera->priv->fill_pattern_data = fill_pattern_data;
@@ -263,17 +257,13 @@ arv_fake_camera_set_fill_pattern (ArvFakeCamera *camera,
 		camera->priv->fill_pattern_callback = arv_fake_camera_diagonal_ramp;
 		camera->priv->fill_pattern_data = NULL;
 	}
-#if GLIB_CHECK_VERSION(2,32,0)
+
 	g_mutex_unlock (&camera->priv->fill_pattern_mutex);
-#else
-	g_mutex_unlock (camera->priv->fill_pattern_mutex);
-#endif
 }
 
 void
 arv_fake_camera_fill_buffer (ArvFakeCamera *camera, ArvBuffer *buffer, guint32 *packet_size)
 {
-	struct timespec time;
 	guint32 width;
 	guint32 height;
 	guint32 exposure_time_us;
@@ -284,8 +274,6 @@ arv_fake_camera_fill_buffer (ArvFakeCamera *camera, ArvBuffer *buffer, guint32 *
 	if (camera == NULL || buffer == NULL)
 		return;
 
-	clock_gettime (CLOCK_MONOTONIC, &time);
-
 	width = _get_register (camera, ARV_FAKE_CAMERA_REGISTER_WIDTH);
 	height = _get_register (camera, ARV_FAKE_CAMERA_REGISTER_HEIGHT);
 	payload = width * height;
@@ -295,30 +283,25 @@ arv_fake_camera_fill_buffer (ArvFakeCamera *camera, ArvBuffer *buffer, guint32 *
 		return;
 	}
 
-	buffer->priv->gvsp_payload_type = ARV_GVSP_PAYLOAD_TYPE_IMAGE;
+	buffer->priv->payload_type = ARV_BUFFER_PAYLOAD_TYPE_IMAGE;
+	buffer->priv->chunk_endianness = G_BIG_ENDIAN;
 	buffer->priv->width = width;
 	buffer->priv->height = height;
 	buffer->priv->status = ARV_BUFFER_STATUS_SUCCESS;
-	buffer->priv->timestamp_ns = ((guint64) time.tv_sec) * 1000000000LL + time.tv_nsec;
+	buffer->priv->timestamp_ns = g_get_real_time () * 1000;
 	buffer->priv->system_timestamp_ns = buffer->priv->timestamp_ns;
 	buffer->priv->frame_id = camera->priv->frame_id++;
 	buffer->priv->pixel_format = _get_register (camera, ARV_FAKE_CAMERA_REGISTER_PIXEL_FORMAT);
 
-#if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_lock (&camera->priv->fill_pattern_mutex);
-#else
-	g_mutex_lock (camera->priv->fill_pattern_mutex);
-#endif
+
 	arv_fake_camera_read_register (camera, ARV_FAKE_CAMERA_REGISTER_EXPOSURE_TIME_US, &exposure_time_us);
 	arv_fake_camera_read_register (camera, ARV_FAKE_CAMERA_REGISTER_GAIN_RAW, &gain);
 	arv_fake_camera_read_register (camera, ARV_FAKE_CAMERA_REGISTER_PIXEL_FORMAT, &pixel_format);
 	camera->priv->fill_pattern_callback (buffer, camera->priv->fill_pattern_data,
 					     exposure_time_us, gain, pixel_format);
-#if GLIB_CHECK_VERSION(2,32,0)
+
 	g_mutex_unlock (&camera->priv->fill_pattern_mutex);
-#else
-	g_mutex_unlock (camera->priv->fill_pattern_mutex);
-#endif
 
 	if (packet_size != NULL)
 		*packet_size = _get_register (camera, ARV_GVBS_STREAM_CHANNEL_0_PACKET_SIZE_OFFSET);
@@ -420,9 +403,9 @@ const char *
 arv_get_fake_camera_genicam_xml (size_t *size)
 {
 	static GMappedFile *genicam_file = NULL;
-	ARV_DEFINE_STATIC_MUTEX (mutex);
+	static GMutex mutex;
 
-	arv_g_mutex_lock (&mutex);
+	g_mutex_lock (&mutex);
 
 	if (genicam_file == NULL ) {
 		char *filename;
@@ -443,7 +426,7 @@ arv_get_fake_camera_genicam_xml (size_t *size)
 		g_free (filename);
 	}
 
-	arv_g_mutex_unlock (&mutex);
+	g_mutex_unlock (&mutex);
 
 	g_return_val_if_fail( genicam_file != NULL, NULL);
 
@@ -470,11 +453,7 @@ arv_fake_camera_new (const char *serial_number)
 
 	memory = g_malloc0 (ARV_FAKE_CAMERA_MEMORY_SIZE);
 
-#if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_init (&fake_camera->priv->fill_pattern_mutex);
-#else
-	fake_camera->priv->fill_pattern_mutex = g_mutex_new ();
-#endif
 	fake_camera->priv->fill_pattern_callback = arv_fake_camera_diagonal_ramp;
 	fake_camera->priv->fill_pattern_data = NULL;
 
@@ -548,11 +527,7 @@ arv_fake_camera_finalize (GObject *object)
 
 	g_free (fake_camera->priv->memory);
 
-#if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_clear (&fake_camera->priv->fill_pattern_mutex);
-#else
-	g_mutex_free (fake_camera->priv->fill_pattern_mutex);
-#endif
 
 	parent_class->finalize (object);
 }
@@ -562,11 +537,17 @@ arv_fake_camera_class_init (ArvFakeCameraClass *fake_camera_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (fake_camera_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (fake_camera_class, sizeof (ArvFakeCameraPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (fake_camera_class);
 
 	object_class->finalize = arv_fake_camera_finalize;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvFakeCamera, arv_fake_camera, G_TYPE_OBJECT)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvFakeCamera, arv_fake_camera, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvFakeCamera))
+#endif
