@@ -28,6 +28,7 @@
 #include <arvgcport.h>
 #include <arvgcregisterdescriptionnode.h>
 #include <arvdevice.h>
+#include <arvgvdevice.h>
 #include <arvchunkparserprivate.h>
 #include <arvbuffer.h>
 #include <arvgcpropertynode.h>
@@ -77,7 +78,7 @@ _pre_remove_child (ArvDomNode *self, ArvDomNode *child)
 /* ArvGcPort implementation */
 
 static gboolean
-_register_workaround_check (ArvGcPort *port, guint64 length)
+_use_legacy_endianess_mechanism (ArvGcPort *port, guint64 length)
 {
 	ArvDomDocument *document;
 	ArvGcRegisterDescriptionNode *register_description;
@@ -105,8 +106,8 @@ arv_gc_port_read (ArvGcPort *port, void *buffer, guint64 address, guint64 length
 		device = arv_gc_get_device (genicam);
 		if (ARV_IS_DEVICE (device)) {
 			/* For schema < 1.1.0 and length == 4, register read must be used instead of memory read.
-			 * See Appendix 3 of Genicam 2.0 specification. */
-			if (_register_workaround_check (port, length)) {
+			 * Only applies to GigE Vision devices. See Appendix 3 of Genicam 2.0 specification. */
+			if (ARV_IS_GV_DEVICE (device) && _use_legacy_endianess_mechanism (port, length)) {
 				guint32 value;
 
 				/* For schema < 1.1.0, all registers are big endian. */
@@ -167,8 +168,8 @@ arv_gc_port_write (ArvGcPort *port, void *buffer, guint64 address, guint64 lengt
 
 		if (ARV_IS_DEVICE (device)) {
 			/* For schema < 1.1.0 and length == 4, register write must be used instead of memory write.
-			 * See Appendix 3 of Genicam 2.0 specification. */
-			if (_register_workaround_check (port, length)) {
+			 * Only applies to GigE Vision devices. See Appendix 3 of Genicam 2.0 specification. */
+			if (ARV_IS_GV_DEVICE (device) && _use_legacy_endianess_mechanism (port, length)) {
 				guint32 value;
 
 				/* For schema < 1.1.0, all registers are big endian. */
@@ -240,7 +241,9 @@ arv_gc_port_class_init (ArvGcPortClass *this_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (this_class);
 	ArvDomNodeClass *dom_node_class = ARV_DOM_NODE_CLASS (this_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (this_class, sizeof (ArvGcPortPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (this_class);
 
@@ -250,4 +253,8 @@ arv_gc_port_class_init (ArvGcPortClass *this_class)
 	dom_node_class->pre_remove_child = _pre_remove_child;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvGcPort, arv_gc_port, ARV_TYPE_GC_FEATURE_NODE)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvGcPort, arv_gc_port, ARV_TYPE_GC_FEATURE_NODE, G_ADD_PRIVATE(ArvGcPort))
+#endif
