@@ -103,6 +103,8 @@ arv_gc_property_node_get_node_name (ArvDomNode *node)
 			return "CommandValue";
 		case ARV_GC_PROPERTY_NODE_TYPE_CHUNK_ID:
 			return "ChunkID";
+		case ARV_GC_PROPERTY_NODE_TYPE_VALUE_DEFAULT:
+			return "ValueDefault";
 
 		case ARV_GC_PROPERTY_NODE_TYPE_P_FEATURE:
 			return "pFeature";
@@ -130,6 +132,8 @@ arv_gc_property_node_get_node_name (ArvDomNode *node)
 			return "pVariable";
 		case ARV_GC_PROPERTY_NODE_TYPE_P_COMMAND_VALUE:
 			return "pCommandValue";
+		case ARV_GC_PROPERTY_NODE_TYPE_P_VALUE_DEFAULT:
+			return "pValueDefault";
 
 		default:
 			return "Unknown";
@@ -332,7 +336,6 @@ arv_gc_property_node_get_int64 (ArvGcPropertyNode *node, GError **error)
 	if (pvalue_node == NULL)
 		return g_ascii_strtoll (_get_value_data (node), NULL, 0);
 
-
 	if (ARV_IS_GC_INTEGER (pvalue_node)) {
 		GError *local_error = NULL;
 		gint64 value;
@@ -398,15 +401,9 @@ arv_gc_property_node_get_double (ArvGcPropertyNode *node, GError **error)
 
 
 	if (ARV_IS_GC_FLOAT (pvalue_node)) {
-		GError *local_error = NULL;
-		double value;
-
-		value = arv_gc_float_get_value (ARV_GC_FLOAT (pvalue_node), &local_error);
-
-		if (local_error != NULL)
-			g_propagate_error (error, local_error);
-
-		return value;
+		return arv_gc_float_get_value (ARV_GC_FLOAT (pvalue_node), error);
+	} else if (ARV_IS_GC_INTEGER (pvalue_node)) {
+		return arv_gc_integer_get_value (ARV_GC_INTEGER (pvalue_node), error);
 	}
 
 	arv_warning_genicam ("[GcPropertyNode::get_double] Invalid node '%s'",
@@ -433,13 +430,10 @@ arv_gc_property_node_set_double (ArvGcPropertyNode *node, double v_double, GErro
 	}
 
 	if (ARV_IS_GC_FLOAT (pvalue_node)) {
-		GError *local_error = NULL;
-
-		arv_gc_float_set_value (ARV_GC_FLOAT (pvalue_node), v_double, &local_error);
-
-		if (local_error != NULL)
-			g_propagate_error (error, local_error);
-
+		arv_gc_float_set_value (ARV_GC_FLOAT (pvalue_node), v_double, error);
+		return;
+	} else if (ARV_IS_GC_INTEGER (pvalue_node)) {
+		arv_gc_integer_set_value (ARV_GC_INTEGER (pvalue_node), v_double, error);
 		return;
 	}
 
@@ -725,6 +719,18 @@ arv_gc_property_node_new_chunk_id (void)
 	return arv_gc_property_node_new (ARV_GC_PROPERTY_NODE_TYPE_CHUNK_ID);
 }
 
+ArvGcNode *
+arv_gc_property_node_new_value_default (void)
+{
+	return arv_gc_property_node_new (ARV_GC_PROPERTY_NODE_TYPE_VALUE_DEFAULT);
+}
+
+ArvGcNode *
+arv_gc_property_node_new_p_value_default (void)
+{
+	return arv_gc_property_node_new (ARV_GC_PROPERTY_NODE_TYPE_P_VALUE_DEFAULT);
+}
+
 static void
 arv_gc_property_node_init (ArvGcPropertyNode *gc_property_node)
 {
